syntax = "proto3";

package xray.transport.internet.headers.wechat;
option csharp_namespace = "Xray.Transport.Internet.Headers.Wechat";
option go_package = "github.com/xtls/xray-core/transport/internet/headers/wechat";
option java_package = "com.xray.transport.internet.headers.wechat";
option java_multiple_files = true;

message VideoConfig {}
