syntax = "proto3";

package xray.proxy.tun;
option csharp_namespace = "Xray.Proxy.Tun";
option go_package = "github.com/xtls/xray-core/proxy/tun";
option java_package = "com.xray.proxy.tun";
option java_multiple_files = true;

message Config {
  string name = 1;
  uint32 MTU = 2;
  uint32 user_level = 3;
}
