// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.1
// source: transport/internet/hysteria/config.proto

package hysteria

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Config struct {
	state                   protoimpl.MessageState `protogen:"open.v1"`
	Version                 int32                  `protobuf:"varint,1,opt,name=version,proto3" json:"version,omitempty"`
	Auth                    string                 `protobuf:"bytes,2,opt,name=auth,proto3" json:"auth,omitempty"`
	Up                      uint64                 `protobuf:"varint,3,opt,name=up,proto3" json:"up,omitempty"`
	Down                    uint64                 `protobuf:"varint,4,opt,name=down,proto3" json:"down,omitempty"`
	Ports                   []uint32               `protobuf:"varint,5,rep,packed,name=ports,proto3" json:"ports,omitempty"`
	Interval                int64                  `protobuf:"varint,6,opt,name=interval,proto3" json:"interval,omitempty"`
	InitStreamReceiveWindow uint64                 `protobuf:"varint,7,opt,name=init_stream_receive_window,json=initStreamReceiveWindow,proto3" json:"init_stream_receive_window,omitempty"`
	MaxStreamReceiveWindow  uint64                 `protobuf:"varint,8,opt,name=max_stream_receive_window,json=maxStreamReceiveWindow,proto3" json:"max_stream_receive_window,omitempty"`
	InitConnReceiveWindow   uint64                 `protobuf:"varint,9,opt,name=init_conn_receive_window,json=initConnReceiveWindow,proto3" json:"init_conn_receive_window,omitempty"`
	MaxConnReceiveWindow    uint64                 `protobuf:"varint,10,opt,name=max_conn_receive_window,json=maxConnReceiveWindow,proto3" json:"max_conn_receive_window,omitempty"`
	MaxIdleTimeout          int64                  `protobuf:"varint,11,opt,name=max_idle_timeout,json=maxIdleTimeout,proto3" json:"max_idle_timeout,omitempty"`
	KeepAlivePeriod         int64                  `protobuf:"varint,12,opt,name=keep_alive_period,json=keepAlivePeriod,proto3" json:"keep_alive_period,omitempty"`
	DisablePathMtuDiscovery bool                   `protobuf:"varint,13,opt,name=disable_path_mtu_discovery,json=disablePathMtuDiscovery,proto3" json:"disable_path_mtu_discovery,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_transport_internet_hysteria_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_transport_internet_hysteria_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_transport_internet_hysteria_config_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetVersion() int32 {
	if x != nil {
		return x.Version
	}
	return 0
}

func (x *Config) GetAuth() string {
	if x != nil {
		return x.Auth
	}
	return ""
}

func (x *Config) GetUp() uint64 {
	if x != nil {
		return x.Up
	}
	return 0
}

func (x *Config) GetDown() uint64 {
	if x != nil {
		return x.Down
	}
	return 0
}

func (x *Config) GetPorts() []uint32 {
	if x != nil {
		return x.Ports
	}
	return nil
}

func (x *Config) GetInterval() int64 {
	if x != nil {
		return x.Interval
	}
	return 0
}

func (x *Config) GetInitStreamReceiveWindow() uint64 {
	if x != nil {
		return x.InitStreamReceiveWindow
	}
	return 0
}

func (x *Config) GetMaxStreamReceiveWindow() uint64 {
	if x != nil {
		return x.MaxStreamReceiveWindow
	}
	return 0
}

func (x *Config) GetInitConnReceiveWindow() uint64 {
	if x != nil {
		return x.InitConnReceiveWindow
	}
	return 0
}

func (x *Config) GetMaxConnReceiveWindow() uint64 {
	if x != nil {
		return x.MaxConnReceiveWindow
	}
	return 0
}

func (x *Config) GetMaxIdleTimeout() int64 {
	if x != nil {
		return x.MaxIdleTimeout
	}
	return 0
}

func (x *Config) GetKeepAlivePeriod() int64 {
	if x != nil {
		return x.KeepAlivePeriod
	}
	return 0
}

func (x *Config) GetDisablePathMtuDiscovery() bool {
	if x != nil {
		return x.DisablePathMtuDiscovery
	}
	return false
}

var File_transport_internet_hysteria_config_proto protoreflect.FileDescriptor

const file_transport_internet_hysteria_config_proto_rawDesc = "" +
	"\n" +
	"(transport/internet/hysteria/config.proto\x12 xray.transport.internet.hysteria\"\x87\x04\n" +
	"\x06Config\x12\x18\n" +
	"\aversion\x18\x01 \x01(\x05R\aversion\x12\x12\n" +
	"\x04auth\x18\x02 \x01(\tR\x04auth\x12\x0e\n" +
	"\x02up\x18\x03 \x01(\x04R\x02up\x12\x12\n" +
	"\x04down\x18\x04 \x01(\x04R\x04down\x12\x14\n" +
	"\x05ports\x18\x05 \x03(\rR\x05ports\x12\x1a\n" +
	"\binterval\x18\x06 \x01(\x03R\binterval\x12;\n" +
	"\x1ainit_stream_receive_window\x18\a \x01(\x04R\x17initStreamReceiveWindow\x129\n" +
	"\x19max_stream_receive_window\x18\b \x01(\x04R\x16maxStreamReceiveWindow\x127\n" +
	"\x18init_conn_receive_window\x18\t \x01(\x04R\x15initConnReceiveWindow\x125\n" +
	"\x17max_conn_receive_window\x18\n" +
	" \x01(\x04R\x14maxConnReceiveWindow\x12(\n" +
	"\x10max_idle_timeout\x18\v \x01(\x03R\x0emaxIdleTimeout\x12*\n" +
	"\x11keep_alive_period\x18\f \x01(\x03R\x0fkeepAlivePeriod\x12;\n" +
	"\x1adisable_path_mtu_discovery\x18\r \x01(\bR\x17disablePathMtuDiscoveryB\x82\x01\n" +
	"$com.xray.transport.internet.hysteriaP\x01Z5github.com/xtls/xray-core/transport/internet/hysteria\xaa\x02 Xray.Transport.Internet.Hysteriab\x06proto3"

var (
	file_transport_internet_hysteria_config_proto_rawDescOnce sync.Once
	file_transport_internet_hysteria_config_proto_rawDescData []byte
)

func file_transport_internet_hysteria_config_proto_rawDescGZIP() []byte {
	file_transport_internet_hysteria_config_proto_rawDescOnce.Do(func() {
		file_transport_internet_hysteria_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_transport_internet_hysteria_config_proto_rawDesc), len(file_transport_internet_hysteria_config_proto_rawDesc)))
	})
	return file_transport_internet_hysteria_config_proto_rawDescData
}

var file_transport_internet_hysteria_config_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_transport_internet_hysteria_config_proto_goTypes = []any{
	(*Config)(nil), // 0: xray.transport.internet.hysteria.Config
}
var file_transport_internet_hysteria_config_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_transport_internet_hysteria_config_proto_init() }
func file_transport_internet_hysteria_config_proto_init() {
	if File_transport_internet_hysteria_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_transport_internet_hysteria_config_proto_rawDesc), len(file_transport_internet_hysteria_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_transport_internet_hysteria_config_proto_goTypes,
		DependencyIndexes: file_transport_internet_hysteria_config_proto_depIdxs,
		MessageInfos:      file_transport_internet_hysteria_config_proto_msgTypes,
	}.Build()
	File_transport_internet_hysteria_config_proto = out.File
	file_transport_internet_hysteria_config_proto_goTypes = nil
	file_transport_internet_hysteria_config_proto_depIdxs = nil
}
