syntax = "proto3";

package xray.transport.internet.hysteria;
option csharp_namespace = "Xray.Transport.Internet.Hysteria";
option go_package = "github.com/xtls/xray-core/transport/internet/hysteria";
option java_package = "com.xray.transport.internet.hysteria";
option java_multiple_files = true;

message Config {
  int32 version = 1;
  string auth = 2;
  uint64 up = 3;
  uint64 down = 4;
  repeated uint32 ports = 5;
  int64 interval = 6;

  uint64 init_stream_receive_window = 7;
  uint64 max_stream_receive_window = 8;
  uint64 init_conn_receive_window = 9;
  uint64 max_conn_receive_window = 10;
  int64 max_idle_timeout = 11;
  int64 keep_alive_period = 12;
  bool disable_path_mtu_discovery = 13;
}

